/*-
 * Copyright (c) 2000 Andrey Simonenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char rcsid[] =
  "@(#)$Id: debug.c,v 1.1.2.4 2000/12/31 18:13:07 simon Exp $";
#endif /* !lint */

#include <syslog.h>
#include <stdarg.h>

#include "debug.h"


int	debug_ipfil, debug_exec, debug_limit, debug_time, debug_lock;
int	debug_ipfil_bkp, debug_exec_bkp, debug_limit_bkp, debug_time_bkp,
	debug_lock_bkp;

#ifdef __FreeBSD__
int	debug_ipfw = 0;
int	debug_ipfw_bkp;
#endif

/*
 * Reset all debug variables (i.e. set them to default value).
 */
void
reset_debug(void)
{
#ifdef __FreeBSD__
	debug_ipfw =
#endif
	debug_ipfil = debug_exec = debug_limit = debug_time = debug_lock = 0;
}

/*
 * Backup current values of debug_* parameters.
 */
void
backup_debug(void)
{
#define BACKUP(param)	param ## _bkp = param

#ifdef __FreeBSD__
	BACKUP(debug_ipfw);
#endif
	BACKUP(debug_ipfil);
	BACKUP(debug_exec);
	BACKUP(debug_limit);
	BACKUP(debug_time);
	BACKUP(debug_lock);
}

/*
 * Restore values of backup_* parameters from backuped copies.
 */
void
restore_debug(void)
{
#define RESTORE(param)	param = param ## _bkp

#ifdef __FreeBSD__
	RESTORE(debug_ipfw);
#endif
	RESTORE(debug_ipfil);
	RESTORE(debug_exec);
	RESTORE(debug_limit);
	RESTORE(debug_time);
	RESTORE(debug_lock);
}