/*-
 * Copyright (c) 2000 Andrey Simonenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *   @(#)$Id: kipfil.h,v 1.1.2.3 2001/01/02 11:55:29 simon Exp $
 */


#ifndef IPA_KIPFIL_H
#define IPA_KIPFIL_H

#include <limits.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#if defined __FreeBSD__
# include <netinet/ip_compat.h>
#elif defined __OpenBSD__
# include <netinet/ip_fil_compat.h>
#endif
#include <netinet/ip_fil.h>

/*
 * XXX Macro U_QUAD_T is defined in <netinet/ip_compat.h> or
 *     <netinet/ip_fil_compat.h> and it can be u_quad_t or u_long.
 */

#ifndef U_QUAD_T
# error Macro U_QUAD_T is not defined.
#endif

#ifdef USE_QUAD_T
# define U_QUAD_T_MAX	UQUAD_MAX
#else
# define U_QUAD_T_MAX	ULONG_MAX
#endif

struct kipfil {
	u_short		group;		/* group number */
	char		type;		/* 'i' or 'o' */
	U_QUAD_T	*bcnt;		/* array of byte counters */
	u_int		size;		/* size of previous array */
	u_int		next_set;	/* next free entry in bcnt array */
};

extern struct kipfil	*kipfil;
extern u_int		nkipfil;
extern int		kipfil_init(void);
extern void		kipfil_close(void);
extern int		kipfil_read_table(void);


#endif /* IPA_KIPFIL_H */
