/*-
 * Copyright (c) 1999-2000 Andrey Simonenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *   @(#)$Id: rules.h,v 1.1.2.7 2000/12/31 18:13:08 simon Exp $
 */

#ifndef IPA_RULES_H
#define IPA_RULES_H


#include "cmd.h"
#include "kipfil.h"


#define ADD	  1
#define SUB	(-1)

/*
 * In original structure ip_fw there is only one field for rule number.
 * I add extra field subnumber: if there are some rules with the same number
 * I distinguish such rules by subnumbers, first rule in the set of such
 * rules has subnubner equal to 0. 
 */
struct ipfwac {
	u_short		number;		/* ipfw rule number */
	u_int		subnumber;	/* ipfw rule subnumber */
	short		set;		/* ==1 if this rule has been already set in ipfw kernel table */
	short		action;		/* subtract or add */
	u_quad_t	bcnt_old;	/* old value of fw_bcnt (struct ip_fw) */
};

struct ipfilac_list {
	u_int		number;		/* number in group */
	short		set;		/* ==1 if this rule has been already set in ipfil kernel table */
	short		action;		/* subtract or add */
	U_QUAD_T	bcnt_old;	/* old value of fr_bytes (struct frentry) */
};

struct ipfilac {
	u_short		group;		/* group number */
	char		type;		/* 'i' or 'o' */
	struct ipfilac_list *list;	/* list of numbers in this group of this type */
	u_int		nlist;		/* size of prev. array */
};

#define UPTO_NOTSET	'-'
#define UPTO_SIMPLE	' '
#define UPTO_MINUTE	'm'
#define UPTO_HOUR	'h'
#define UPTO_dAY	'd'
#define UPTO_DAY	'D'
#define UPTO_wEEK	'w'
#define UPTO_WEEK	'W'
#define UPTO_MONTH	'M'

struct time_exp {
	time_t		seconds;	/* number of seconds */
	u_int		monthes;	/* number of monthes */
	char		upto;		/* m, h, D, W, M, - (not set) */
	short		side;		/* == 0 if +X is set on the left side,
					   == 1 if +X if set on the right side */
};

struct interval {
	int		h1, m1, h2, m2;	/* h1:m1 - h2:m2 */
};

struct worktime {
	struct interval	*interval;	/* ptr. to interval list */
	u_int		ninterval;	/* size of prev. array */
};

struct expire {
	struct time_exp	time;		/* "expire_time" parameter */
	struct cmd	*cmd;		/* "exec" parameters */
	u_int		ncmd;		/* size of prev. array */
};

struct limit {
	FILE		*fp;		/* ptr. on FILE stream for limit db */
	int		fd;		/* == fileno(fp) */
	char		*filename;	/* name of limit db file */
	char		*limitname;	/* name of limit */
	u_quad_t	byte_limit;	/* "byte_limit" parameter */
	u_quad_t	bcnt;		/* byte counter for this limit */
	u_quad_t	bcnt_sub;	/* number of bytes which should be subtracted
					   from limit counter, isn' equal to 0
					   if ``wrong'' configuration */
	struct cmd	*cmd;		/* "expire" section with "exec" parameters */
	u_int		ncmd;		/* size of prev. array */
	struct time_exp	zero_time_param;/* "zero_time" parameter */
	struct expire	expire;		/* "expire" section */
	struct commands	rc[2];		/* rc[0] -- "startup", rc[1] -- "shutdown" section */
	char		*info;		/* "info" parameter */
	pid_t		pid;		/* PID of background process which
					   runs "exec" parameter commands */
	u_short		is_changed;	/* ==1 if bcnt was modified */
	u_short		status;		/* status */
	time_t		wakeup;
	struct worktime	worktime[7];	/* "worktime" parameter */
	short		use_worktime;	/* ==1 if "worktime" parameter was used */
	time_t		start_time;	/* time when limit was started */
	time_t		zero_time;	/* time when limit should be zeroed */
	time_t		reach_time;	/* time when limit was reached */
	time_t		expire_time;	/* time when limit will be expired */
};

#define	LIMIT_IS_REACHED	0x0001
#define LIMIT_EXEC		0x0002
#define LIMIT_EXPIRE_CHECK	0x0004
#define LIMIT_EXPIRE_EXEC	0x0008


struct ipac {
	char		*rulename;	/* name of rule */
	u_int		update_db_time;	/* interval in sec. when to update counters,
					   "update_db_time" parameter */
	u_int		append_db_time;	/* interval in sec. when to append new record in data base.
					   "append_db_time" parameter */
	time_t		wakeup;		/* time in sec. when to wake up */
	time_t		newrec_time;	/* time when to append new record in data base */
	FILE		*fp;		/* ptr. on FILE stream for db file */
	int		fd;		/* == fileno(fp) */
	char		*filename;	/* name of db file */
	u_quad_t	bcnt;		/* byte counter for this rule */
	u_quad_t	bcnt_sub;	/* nuber of bytes which should be subtracted
					   from accounting, isn't equal to 0
					   if ``wrong'' configuration */
	u_quad_t	maxchunk;	/* "maxchunk" parameter */
#ifdef __FreeBSD__
	struct ipfwac	*ipfwac;	/* ptr. on array for ipfw accounting */
	u_int		nipfwac;	/* size of prev. array */
#endif
	struct ipfilac	*ipfilac;	/* ptr. on array for ipfil accounting */
	u_int		nipfilac;	/* size of prev. array */
	struct limit	*limit;		/* ptr. on array of limits */
	u_int		nlimit;		/* szie of prev. array */
	struct commands	rc[2];		/* rc[0] -- "startup", rc[1] -- "shutdown" section */
	char		*acl;		/* "acl" parameter */
	char		*info;		/* "info" parameter */
	struct worktime	worktime[7];	/* "worktime" parameter */
	short		use_worktime;	/* ==1 if worktime parameter is used */
};


extern struct ipac	* ipac_new();
extern struct ipac	*ipac;
extern u_int		nipac;
extern int		use_ipfw, use_ipfil;
extern char		*acl_global;
extern short		use_worktime_global;
extern struct worktime	worktime_global[7];
extern struct commands	startup_global, shutdown_global;
extern int		run_ipac(void);
extern int		build_db_regexes(void);
extern void		end_work(int), sig_alrm(int), reconfigure(int), sig_chld(int);
extern void		free_cmd_list(struct cmd *, u_int);
extern int		run_rules_rc(int);
extern int		init_limits(int);


#endif /* !IPA_RULES_H */
